<?php
// $Id: erp.inc,v 1.23.2.9 2009/11/16 07:39:02 simon Exp $

/**
 * ERP common helper module
 */

/**
 * Shrink text
 *
 * @param $text
 * @param $shrink
 * @return unknown_type
 */
function shrink($text, $shrink = 0) {
  if (strlen($text) > $shrink + 2) {
    return substr($text, 0, $shrink) .'..';
  }
  else {
    return ($text ? $text : '');
  }
}

/**
 * Validate a job/timesheet/whatever's owner
 *
 * @param unknown_type $node
 * @return unknown
 */
function erp_owner_validate($name) {
  // Try and retrieve the uid for the job owner based on the name
  $owner = user_load(array('name' => $name));

  return ($owner->uid ? $owner->uid : 0);
}

function erp_oid_validate($oid) {
  // Try and retrieve the owner name from the uid
  $owner = user_load(array('uid' => $oid));

  return ($owner->name ? $owner->name : '');
}

/**
 * Function used to both display and set the next invoice number,
 * po number, etc. see invoice_settings, purchase_order_settings
 * Defaults to collapsed as its not common
 *
 * @param unknown_type $type
 * @param unknown_type $desc
 * @return unknown
 */
function erp_set_next_id_form($table, $desc) {
  $type_id = db_result(db_query("SELECT auto_increment FROM information_schema.tables WHERE table_name = '%s' and auto_increment is not null", $table));

  $form[$table .'_settings']['set_next'] =
    array('#type' => 'fieldset',
          '#title' => t('Next ') . $desc . t(' number'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#description' => t('You can set the number for the next created ') . $desc . t(' here.'));

  $form[$table .'_settings']['set_next']['number'] =
    array('#type' => 'textfield',
          '#size' => 10,
          '#maxlength' => 10,
          '#title' => t('Next ') . $desc . t(' number'),
          '#default_value' => $type_id);

  $form[$table .'_settings']['set_next']['table'] =
    array('#type' => 'hidden',
          '#value' => $table);

  $form[$table .'_settings']['set_next']['button'] =
    array('#type' => 'submit',
          '#value' => t('Set number'));

  $form['#submit'][] = 'erp_set_next_id_submit';

  return $form;
}

/**
 * Tiny submit handler for erp_set_next_id_form
 *
 * @param unknown_type $form
 * @param unknown_type $form_state
 */
function erp_set_next_id_submit($form, &$form_state) {
	if ($form_state['clicked_button']['#value'] == t('Set number')) {
    db_query("ALTER TABLE {%s} AUTO_INCREMENT = %d", $form_state['values']['table'], $form_state['values']['number']);
  }
}

/**
 * This is is common function used by quote, invoice, cash_sale, purchase_order, and so is
 * placed here. It does depend on "erp_item" being enabled however, meaning all those depend
 * on erp_item as well.
 *
 */
function erp_add_newline(&$node, &$form_state) {
  if ($form_state['newline']) {
  	$newline = $form_state['newline'];
    if (!isset($node->entries)) {
    	$node->entries = array();
    }
    $node->entries['qty'][] = $newline['qty'];
    $node->entries['item'][] = $newline['item'];
    $node->entries['item_desc'][] = $newline['item_desc'];
    $node->entries['extra'][] = $newline['extra'];
    $node->entries['serial'][] = $newline['serial'];
    $node->entries['completed_date'][] = $newline['completed_date'];
    $node->entries['price'][] = $newline['price'];
  }

  $remove_delta = -1;
  if (!empty($form_state['remove_delta'])) {
  	$remove_delta = $form_state['remove_delta'] - 1;
	  for ($i = 0; $i < count($node->entries['qty']); $i++) {
	    if ($i == $remove_delta) {
	      foreach ($node->entries as $value => $key) {
	        if (is_array($node->entries[$value])) {
	          erp_delete_array_key($node->entries[$value], $i);
	        }
	      }
	    }
	  }
  }
}

function erp_newline_fields_validate($form_state) {
  if (!is_array($form_state['values']['entries']['qty']) && !$form_state['values']['entries']['newline']) {
    form_set_error('newline', t('You need to enter at least one "line" before submitting.'));
  }
  else {
    for ($i = 0; $i < count($form_state['values']['entries']['qty']); $i++) {
      if ($form_state['values']['entries']['item_desc'][$i]) {
        $item_info = erp_item_desc_breakup($form_state['values']['entries']['item_desc'][$i]);
        $supplier_id = $item_info[0];
        $stock_code = $item_info[1];

        $item_nid = erp_item_locate($supplier_id, $stock_code, FALSE);
        if (!$item_nid) {
          form_set_error('entries][item_desc]['. $i, t('Invalid stock code'));
        }
      }
      // @todo: Check serial numbers are actually still available for sale
    }
  }
}

function erp_add_newline_submit($form, &$form_state) {
	$form_state['remove_delta'] = 0;

  // Set the form to rebuild and run submit handlers.
  node_form_submit_build_node($form, $form_state);

  // Make the changes we want to the form state.
  if ($form_state['values']['addline']) {
  	list($supplier_id, $stock_code, $desc, $price) = erp_item_desc_breakup($form_state['values']['entries']['newline']['item_desc']);
  	$item_nid = erp_item_locate($supplier_id, $stock_code, FALSE);

    if ($item_nid) {
    	$item_node = node_load($item_nid);

	    $new_entry['qty'] = $form_state['values']['entries']['newline']['qty'];
	    $new_entry['item'] = $item_nid;
	    $new_entry['item_desc'] = erp_item_desc_build($item_node);
	    $new_entry['extra'] = $form_state['values']['entries']['newline']['extra'];
	    if (variable_get('erp_invoice_serial', FALSE)) {
	      $new_entry['serial'] = $form_state['values']['entries']['newline']['serial'];
	    }
	    $new_entry['completed_date'] = $form_state['values']['entries']['newline']['completed_date'];
	    $new_entry['price'] = $form_state['values']['entries']['newline']['price'];
	    $form_state['newline'] = $new_entry;
  	}
  }
}

function erp_remove_line_submit($form, &$form_state) {
  // Set the form to rebuild and run submit handlers.
  node_form_submit_build_node($form, $form_state);
}

function erp_line_js($delta = 0) {
	$form = erp_ajax_form_handler($delta);

  // Render the new output.
  $erp_form = $form['entries'];

  // Prevent duplicate wrappers.
  unset($erp_form['#prefix'], $erp_form['#suffix']);

  $output = theme('status_messages') . drupal_render($erp_form);

  // Final rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * This does various processes required for ajax form handling
 *
 */
function erp_ajax_form_handler($delta = 0) {
	// The form is generated in an include file which we need to include manually.
  include_once 'modules/node/node.pages.inc';
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];

  // Get the form from the cache.
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);

  // We need to process the form, prepare for that by setting a few internals.
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;

  // Set up our form state variable, needed for removing tracks.
  $form_state['remove_delta'] = $delta;

  if (!isset($_POST['op'])) {
    // Disable #required and #element_validate validation.
    _erp_ahah_helper_disable_validation($form);
  }

  // Build, validate and if possible, submit the form.
  drupal_process_form($form_id, $form, $form_state);

  // If validation fails, force form submission - this is my own "hack" for overcoming
  // issues where all required fields need to be filled out before the 'add more' button
  // can be clicked.  A better solution is being worked on in Drupal's issue queue.
  if (form_get_errors()) {
    form_execute_handlers('submit', $form, $form_state);
  }

  // This call recreates the form relying solely on the form_state that the
  // drupal_process_form set up.
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);

  return $form;
}

function _erp_ahah_helper_disable_validation(&$form) {
  foreach (element_children($form) as $child) {
    $form[$child]['#validated'] = TRUE;
    _erp_ahah_helper_disable_validation($form[$child]);
  }
}

function erp_fix_ahah_action($node) {
  global $base_path;
  if ($node->nid) {
  	$action = $base_path .'node/'. $node->nid .'/edit';
  }
  else {
  	$action = $base_path .'node/add/'. str_replace('_', '-', $node->type);
  }
  return $action;
}

/**
 * Given an array, remove one item, array_splice renumbers as well, which is nice.
 *
 * @param array $array
 * @param int $key
 */
function erp_delete_array_key(&$array, $key) {
  array_splice($array, $key, 1);
}


function build_entries_header($form, $item = 0) {
  $header = array();
  if (isset($form['qty'][$item])) {
    $header[] = t('Qty');
  }
  if (isset($form['item_desc'][$item])) {
    $header[] = t('Stock Code');
  }
  if (isset($form['completed_date'][$item])) {
    $header[] = t('Date');
  }
  if (isset($form['extra'][$item])) {
    $header[] = t('Extra Text');
  }
  if (isset($form['serial'][$item])) {
    $header[] = t('Serial');
  }
  if (isset($form['price'][$item])) {
    $header[] = t('Price');
  }
  if (isset($form['delete'][$item])) {
    $header[] = t('Delete');
  }
  return $header;
}

function erp_get_total($node) {
  $total = 0;
  if ($node->entries) {
    for ($i=0; $i < count($node->entries['qty']); $i++) {
      $total += $node->entries['qty'][$i] * $node->entries['price'][$i];
    }
  }
  return $total;
}

/**
 * If the node does not have a customer set, set the id & text customer now
 *
 * @param unknown_type $node
 */
function erp_set_customer(&$node) {
  if (!is_object($node)) {
    $node = new stdClass();
  }

  if (!$node->customer_nid) {
    if ($_GET['customer']) {
      $node->customer_nid = $_GET['customer'];
    }
    elseif ($_SESSION['curr_customer']) {
      $node->customer_nid = $_SESSION['curr_customer'];
    }
  }

  if ($node->customer_nid && !isset($node->customer)) {
    $customer = node_load($node->customer_nid);
    $node->customer = $customer->title;
  }
}

/**
 * Helper form validate function to remove form_state['storage'] during submit.
 * If it's set, page redirection fails (i.e. doesn't happen).
 */
function erp_clear_storage($form, &$form_state) {
  $form_state['storage'] = NULL;
}

function erp_transfer_data(&$node, &$newnode, $type) {
  node_invoke($node, 'prepare_'. $type);

  $node->comment = $newnode->comment;
  $node->created = $newnode->created;
  $node->date = $newnode->date;
  $node->name = $newnode->name;
  $node->type = $newnode->type;
  $node->uid = $newnode->uid;
  $node->refs = $node->nid;

  // As this is a new node, we need to unset the current node id's
  unset($node->nid);
  unset($node->vid);
  unset($node->title);
  unset($node->comment_count);
  unset($node->printed);

  // Calculate price on buy price when creating invoice from PO
  // Maybe not the safest thing to do - check goods receive? too hard?
  if ($node->type == 'erp_purchase_order') {
    for ($i = 0; $i < count($node->entries['item']); $i++) {
      $price = erp_pricing_get_entry($customer, 'markup') * $node->entries['price'][$i];
      $node->entries['price'][$i] = erp_tax_erp_tax_inc_amount($price);
    }
  }
  else {
	  for ($i = 0; $i < count($node->entries['item']); $i++) {
	    // If no price is set, default to the 'sell' price.
	    if (!isset($node->entries['price'][$i])) {
	      $node->entries['price'][$i] = erp_item_price($node->entries['item'][$i], 'sell');
	    }
	  }
  }
}
